#include "demo.i"
#include "generated.i"

		.global	l_tile
		.global	l_tile_yflip
		.global	gettile
		.global	puttile

		; Each 8-bit tile line is stored as, msb to lsb:
		; {p0, p0^p1, p1^p2, p1^p3, p2^p4, p4^p5, p5^p6, p3^p7}
		; (note the anomalies!)

l_tile_yflip:
		DE99				;	t246

		ldi	r17, 112		; 1
		sub	r17, r15		; 1
		rjmp	common			; 2

l_tile:
		DE101				;	t246

		mov	r17, r15		; 1
		subi	r17, 1			; 1

common:
		lds	r16, yscroll		; 2
		add	r17, r16		; 1

		; r17 is rrrryyyx, r = map coord, y = tile coord, x = don't care

		; End of map at 0x500 - 5 * 19 = 0x4a1

		ldi	r26, 0xa1		; 1
		ldi	r27, 0x04		; 1

		swap	r17			; 1
		mov	r16, r17		; 1
		andi	r16, 15			; 1
		ldi	r18, 19 * 5		; 1
		mul	r16, r18		; 2

		sub	r26, r0			; 1
		sbc	r27, r1			; 1

		; Pick the correct set of colour bytes

		ldi	r18, 19			; 1
		sbrc	r19, 0			; \_ 2
		ldi	r18, 19 * 3		; /
		movw	r28, r26		; 1
		add	r28, r18		; 1
		adc	r29, r3			; 1

		; Locate the tile data

		ldi	r31, hi8(tiles)		; 1
		lsl	r17			; 1
		adc	r31, r3			; 1
		andi	r17, 0xc0		; 1

		; r31:r17 points to start of tile table in flash
		; X points to 19 bytes {tilenumber}
		; X points to 19 byte pairs {bg, bg ^ fg}

		ld	r30, X+			; 2	t372
		or	r30, r17		; 1
		lpm	r16, Z			; 3
		ld	r20, Y+			; 2
		ld	r21, Y+			; 2

		mov	r24, r2			; 1
		ldi	r18, 19			; 1
1:
		out	PORTD, r24		; 1	t384

		ld	r30, X+			; 2
		sbrc	r16, 7			; \_ 2
		eor	r20, r21		; /

		out	PORTD, r20		; 1	t389, first visible pixel

		or	r30, r17		; 1
		sbrc	r16, 6			; \_ 2
		eor	r20, r21		; /
		mov	r24, r20		; 1

		out	PORTD, r20		; 1

		sbrc	r16, 5			; \_ 2
		eor	r20, r21		; /
		sbrc	r16, 4			; \_ 2
		eor	r24, r21		; /

		out	PORTD, r20		; 1

		mov	r25, r16		; 1
		lpm	r16, Z			; 3

		out	PORTD, r24		; 1

		ld	r22, Y+			; 2
		sbrc	r25, 3			; \_ 2
		eor	r20, r21		; /

		out	PORTD, r20		; 1

		ld	r23, Y+			; 2
		sbrc	r25, 2			; \_ 2
		eor	r20, r21		; /

		out	PORTD, r20		; 1

		sbrc	r25, 1			; \_ 2
		eor	r20, r21		; /
		sbrc	r25, 0			; \_ 2
		eor	r24, r21		; /

		out	PORTD, r20		; 1

		movw	r20, r22		; 1
		subi	r18, 1			; 1
		brne	1b			; 1/2

		DE1
		out	PORTD, r24		; 1	t9
		DE8
		out	PORTD, r2		; 1	t18
		rjmp	endofline
 
gettile:
		; Get a tile
		; Input:
		;	r18		X coordinate
		;	X		Pointer to current row
		; Output:
		;	r17		Tile
		;	r22, r23	fg
		;	r24, r25	bg
		; Clobbers:
		;	X

		add	r26, r18
		adc	r27, r3
		ld	r17, X
		add	r26, r18
		adc	r27, r3
		adiw	r26, 19
		ld	r24, X+
		ld	r22, X
		adiw	r26, 37
		ld	r25, X+
		cpi	r17, TILE_BLANK
		brne	1f
		movw	r22, r24
		ret
1:
		ld	r23, X
		eor	r22, r24
		eor	r23, r25
		ret

puttile:
		; Put a tile
		; Input:
		;	r17		Tile
		;	r18		X coordinate
		;	r22, r23	fg
		;	r24, r25	bg
		;	X		Pointer to current row

		; Clobbers:
		;	r22, r23, X

		eor	r22, r24
		eor	r23, r25
		add	r26, r18
		adc	r27, r3
		st	X, r17
		add	r26, r18
		adc	r27, r3
		adiw	r26, 19
		st	X+, r24
		st	X, r22
		adiw	r26, 37
		st	X+, r25
		st	X, r23
		ret

